@@ -1,26 +1,7 @@
-# This file was part of and modifed from Hypothesis, which may be found at
-# https://github.com/HypothesisWorks/hypothesis/
-#
-# Most of this work is copyright (C) 2013-2021 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# ./CONTRIBUTING.rst for a full list of people who may hold copyright,
-# and consult the git log of ./hypothesis-python/src/hypothesis/extra/numpy.py
-# if you need to determine who owns an individual contribution.
-# ('.' represents the root of the Hypothesis git repository)
-#
-# This Source Code Form is subject to the terms of the Mozilla Public License,
-# v. 2.0. If a copy of the MPL was not distributed with this file, You can
-# obtain one at https://mozilla.org/MPL/2.0/.
-
-"""Note this reference page is very much WIP.
-See `Hypothesis for the scientific stack
-<https://hypothesis.readthedocs.io/en/latest/numpy.html#numpy>`_
-for a general idea on what we're doing here.
-"""
-
 import math
 from collections import defaultdict
 from functools import update_wrapper
+from numbers import Real
 from types import SimpleNamespace
 from typing import (Any, Iterable, List, Mapping, NamedTuple, Optional,
                     Sequence, Tuple, Type, TypeVar, Union)
@@ -35,11 +16,12 @@ from hypothesis.strategies._internal.strategies import check_strategy
 
 __all__ = [
     "get_strategies_namespace",
+    "from_dtype",
     "arrays",
     "array_shapes",
-    "from_dtype",
     "scalar_dtypes",
     "boolean_dtypes",
+    "numeric_dtypes",
     "integer_dtypes",
     "unsigned_integer_dtypes",
     "floating_dtypes",
@@ -54,7 +36,8 @@ Boolean = TypeVar("Boolean")
 SignedInteger = TypeVar("SignedInteger")
 UnsignedInteger = TypeVar("UnsignedInteger")
 Float = TypeVar("Float")
-DataType = Union[Boolean, SignedInteger, UnsignedInteger, Float]
+Numeric = Union[SignedInteger, UnsignedInteger, Float]
+DataType = Union[Boolean, Numeric]
 Array = TypeVar("Array")
 Shape = Tuple[int, ...]
 BasicIndex = Tuple[Union[int, slice, None, "ellipsis"], ...]  # noqa: F821
@@ -65,11 +48,12 @@ class BroadcastableShapes(NamedTuple):
     result_shape: Shape
 
 
-SIGNED_INT_NAMES = ["int8", "int16", "int32", "int64"]
-UNSIGNED_INT_NAMES = ["uint8", "uint16", "uint32", "uint64"]
-ALL_INT_NAMES = SIGNED_INT_NAMES + UNSIGNED_INT_NAMES
+INT_NAMES = ["int8", "int16", "int32", "int64"]
+UINT_NAMES = ["uint8", "uint16", "uint32", "uint64"]
+ALL_INT_NAMES = INT_NAMES + UINT_NAMES
 FLOAT_NAMES = ["float32", "float64"]
-DTYPE_NAMES = ["bool"] + ALL_INT_NAMES + FLOAT_NAMES
+NUMERIC_NAMES = ALL_INT_NAMES + FLOAT_NAMES
+DTYPE_NAMES = ["bool"] + NUMERIC_NAMES
 
 
 def partition_attributes_and_stubs(
@@ -89,9 +73,9 @@ def partition_attributes_and_stubs(
 
 def infer_xp_is_compliant(xp):
     try:
-        array = xp.asarray(0, dtype=xp.int8)
+        array = xp.zeros(1)
         array.__array_namespace__()
-    except AttributeError:
+    except Exception:
         warn(
             f"Could not determine whether module {xp.__name__} "
             "is an Array API library",
@@ -104,7 +88,6 @@ def check_xp_attributes(xp, attributes: List[str]):
     for attr in attributes:
         if not hasattr(xp, attr):
             missing_attrs.append(attr)
-
     if len(missing_attrs) > 0:
         f_attrs = ", ".join(missing_attrs)
         raise AttributeError(
@@ -128,30 +111,30 @@ def order_check(name, floor, min_, max_):
         raise InvalidArgument(f"min_{name}={min_} is larger than max_{name}={max_}")
 
 
-def find_dtype_builtin_family(
-        xp, dtype: Type[DataType]
+def find_castable_builtin_for_dtype(
+    xp, dtype: Type[DataType]
 ) -> Tuple[Type[Union[bool, int, float]], List[str]]:
-    builtin_family = None
+    builtin = None
     stubs = []
 
     try:
         bool_dtype = xp.bool
         if dtype == bool_dtype:
-            builtin_family = bool
+            builtin = bool
     except AttributeError:
         stubs.append("bool")
 
     int_dtypes, int_stubs = partition_attributes_and_stubs(xp, ALL_INT_NAMES)
     if dtype in int_dtypes:
-        builtin_family = int
+        builtin = int
     stubs.extend(int_stubs)
 
     float_dtypes, float_stubs = partition_attributes_and_stubs(xp, FLOAT_NAMES)
     if dtype in float_dtypes:
-        builtin_family = float
+        builtin = float
     stubs.extend(float_stubs)
 
-    return builtin_family, stubs
+    return builtin, stubs
 
 
 def dtype_from_name(xp, name: str) -> Type[DataType]:
@@ -182,53 +165,81 @@ def from_dtype(
     exclude_min: Optional[bool] = None,
     exclude_max: Optional[bool] = None,
 ) -> st.SearchStrategy[Union[bool, int, float]]:
-    """Creates a strategy which can generate any castable value of the given
-    Array API dtype."""
+    """Return a strategy for any value of the given dtype.
+
+    Values generated are of the Python scalar which is
+    :array-ref:`promotable <type_promotion.html>` to ``dtype``, where the values
+    do not exceed its bounds.
+
+    * ``dtype`` may be a dtype object or the string name of a
+      :array-ref:`valid dtype <data_types.html>`.
+
+    Compatible ``**kwargs`` are passed to the inferred strategy function for
+    integers and floats.  This allows you to customise the min and max values,
+    and exclude non-finite numbers. This is particularly useful when kwargs are
+    passed through from :func:`arrays`, as it seamlessly handles the ``width``
+    or other representable bounds for you.
+    """
     infer_xp_is_compliant(xp)
     check_xp_attributes(xp, ["iinfo", "finfo"])
 
     if isinstance(dtype, str):
         dtype = dtype_from_name(xp, dtype)
 
-    builtin_family, stubs = find_dtype_builtin_family(xp, dtype)
+    builtin, stubs = find_castable_builtin_for_dtype(xp, dtype)
 
-    if builtin_family is bool:
+    if builtin is None:
+        if len(stubs) > 0:
+            warn_on_missing_dtypes(xp, stubs)
+        raise InvalidArgument(f"No strategy inference for {dtype}")
+
+    if builtin is bool:
         return st.booleans()
 
-    def minmax_values_kw(info):
-        kw = {}
+    def check_min_value(info_obj):
+        assert isinstance(min_value, Real)
+        if min_value < info_obj.min:
+            raise InvalidArgument(
+                f"dtype {dtype} requires min_value={min_value} "
+                f"to be at least {info_obj.min}"
+            )
 
-        if min_value is None:
-            kw["min_value"] = info.min
-        else:
-            if min_value < info.min:
-                raise InvalidArgument(
-                    f"dtype {dtype} requires min_value={min_value} "
-                    f"to be at least {info.min}"
-                )
-            kw["min_value"] = min_value
+    def check_max_value(info_obj):
+        assert isinstance(max_value, Real)
+        if max_value > info_obj.max:
+            raise InvalidArgument(
+                f"dtype {dtype} requires max_value={max_value} "
+                f"to be at most {info_obj.max}"
+            )
 
-        if max_value is None:
-            kw["max_value"] = info.max
-        else:
-            if max_value > info.max:
-                raise InvalidArgument(
-                    f"dtype {dtype} requires max_value={max_value} "
-                    f"to be at most {info.max}"
-                )
-            kw["max_value"] = max_value
-
-        return kw
-
-    if builtin_family is int:
+    if builtin is int:
         iinfo = xp.iinfo(dtype)
-        kw = minmax_values_kw(iinfo)
+        kw = {}
+        if min_value is None:
+            kw["min_value"] = iinfo.min
+        else:
+            check_min_value(iinfo)
+            kw["min_value"] = min_value
+        if max_value is None:
+            kw["max_value"] = iinfo.max
+        else:
+            check_max_value(iinfo)
+            kw["max_value"] = max_value
         return st.integers(**kw)
 
-    if builtin_family is float:
+    if builtin is float:
         finfo = xp.finfo(dtype)
-
-        kw = minmax_values_kw(finfo)
+        kw = {}
+        # Whilst we know the boundary values of float dtypes we do not assign
+        # them to the floats() strategy by default - passing min/max values will
+        # modify test case reduction behaviour so that simple bugs may become
+        # harder for users to identiy.
+        if min_value is not None:
+            check_min_value(finfo)
+            kw["min_value"] = min_value
+        if max_value is not None:
+            check_max_value(finfo)
+            kw["max_value"] = max_value
         if allow_nan is not None:
             kw["allow_nan"] = allow_nan
         if allow_infinity is not None:
@@ -237,14 +248,8 @@ def from_dtype(
             kw["exclude_min"] = exclude_min
         if exclude_max is not None:
             kw["exclude_max"] = exclude_max
-
         return st.floats(width=finfo.bits, **kw)
 
-    if len(stubs) > 0:
-        warn_on_missing_dtypes(xp, stubs)
-
-    raise InvalidArgument(f"No strategy inference for {dtype}")
-
 
 class ArrayStrategy(st.SearchStrategy):
     def __init__(self, xp, elements_strategy, dtype, shape, fill, unique):
@@ -256,8 +261,7 @@ class ArrayStrategy(st.SearchStrategy):
         self.unique = unique
         self.array_size = math.prod(shape)
 
-        builtin_family, _ = find_dtype_builtin_family(xp, dtype)
-        self.builtin = builtin_family
+        self.builtin, _ = find_castable_builtin_for_dtype(xp, dtype)
 
     def set_value(self, result, i, val, strategy=None):
         strategy = strategy or self.elements_strategy
@@ -271,8 +275,7 @@ class ArrayStrategy(st.SearchStrategy):
         self.check_set_value(val, result[i], strategy)
 
     def check_set_value(self, val, val_0d, strategy):
-        if ((val == val and val_0d != val) or
-                (self.xp.isfinite(val_0d) and self.builtin(val_0d) != val)):
+        if self.xp.isfinite(val_0d) and self.builtin(val_0d) != val:
             raise InvalidArgument(
                 f"Generated array element {val!r} from strategy {strategy} "
                 f"cannot be represented as dtype {self.dtype}. "
@@ -284,10 +287,15 @@ class ArrayStrategy(st.SearchStrategy):
 
     def do_draw(self, data):
         if 0 in self.shape:
-            return self.xp.empty(self.shape, dtype=self.dtype)
+            return self.xp.zeros(self.shape, dtype=self.dtype)
 
         if self.fill.is_empty:
-            result = self.xp.empty(self.array_size, dtype=self.dtype)
+            # We have no fill value (either because the user explicitly
+            # disabled it or because the default behaviour was used and our
+            # elements strategy does not produce reusable values), so we must
+            # generate a fully dense array with a freshly drawn value for each
+            # entry.
+            result = self.xp.zeros(self.array_size, dtype=self.dtype)
             if self.unique:
                 seen = set()
                 elements = cu.many(
@@ -299,17 +307,23 @@ class ArrayStrategy(st.SearchStrategy):
                 i = 0
                 while elements.more():
                     val = data.draw(self.elements_strategy)
-                    self.set_value(result, i, val)
-                    if result[i] not in seen:
-                        seen.add(val)
-                        i += 1
-                    else:
+                    if val in seen:
                         elements.reject()
+                    else:
+                        seen.add(val)
+                        self.set_value(result, i, val)
+                        i += 1
             else:
                 for i in range(self.array_size):
                     val = data.draw(self.elements_strategy)
                     self.set_value(result, i, val)
         else:
+            # We draw arrays as "sparse with an offset". We assume not every
+            # element will be assigned and so first draw a single value from our
+            # fill strategy to create a full array. We then draw a collection of
+            # index assignments within the array and assign fresh values from
+            # our elements strategy to those indices.
+
             fill_val = data.draw(self.fill)
             try:
                 result = self.xp.full(self.array_size, fill_val, dtype=self.dtype)
@@ -331,6 +345,9 @@ class ArrayStrategy(st.SearchStrategy):
                 data,
                 min_size=0,
                 max_size=self.array_size,
+                # sqrt isn't chosen for any particularly principled reason. It
+                # just grows reasonably quickly but sublinearly, and for small
+                # arrays it represents a decent fraction of the array size.
                 average_size=math.sqrt(self.array_size),
             )
 
@@ -368,10 +385,79 @@ def arrays(
     fill: Optional[st.SearchStrategy[Any]] = None,
     unique: bool = False,
 ) -> st.SearchStrategy[Array]:
-    """Returns a strategy for generating Array API arrays."""
+    """Returns a strategy for :array-ref:`arrays <array_object.html>`.
+
+    * ``dtype`` may be a :array-ref:`valid dtype <data_types.html>` object or
+      name, or a strategy that generates such values.
+    * ``shape`` may be an integer >= 0, a tuple of such integers, or a strategy
+      that generates such values.
+    * ``elements`` is a strategy for values to put in the array. If ``None``
+      then a suitable value will be inferred based on the dtype, which may give
+      any legal value (including e.g. NaN for floats). If a mapping, it will be
+      passed as ``**kwargs`` to ``from_dtype()`` when inferring based on the dtype.
+    * ``fill`` is a strategy that may be used to generate a single background
+      value for the array. If ``None``, a suitable default will be inferred
+      based on the other arguments. If set to
+      :func:`~hypothesis.strategies.nothing` then filling behaviour will be
+      disabled entirely and every element will be generated independently.
+    * ``unique`` specifies if the elements of the array should all be distinct
+      from one another. Note that in this case multiple NaN values may still be
+      allowed. If fill is also set, the only valid values for fill to return are
+      NaN values.
+
+    Arrays of specified ``dtype`` and ``shape`` are generated for example
+    like this:
+
+    .. code-block:: pycon
+
+      >>> from numpy import array_api as xp
+      >>> arrays(xp, xp.int8, (2, 3)).example()
+      Array([[-8,  6,  3],
+             [-6,  4,  6]], dtype=int8)
+
+    Specifying element boundaries by a :obj:`python:dict` of the kwargs to pass
+    to :func:`from_dtype` will ensure ``dtype`` bounds will be respected.
+
+    .. code-block:: pycon
+
+      >>> arrays(xp, xp.int8, 3, elements={"min_value": 10}).example()
+      Array([125, 13, 79], dtype=int8)
+
+    Refer to :hyp-ref:`What you can generate and how <data.html>` for passing
+    your own elements strategy.
+
+    .. code-block:: pycon
+
+      >>> arrays(xp, xp.float32, 3, elements=floats(0, 1, width=32)).example()
+      Array([ 0.88974794,  0.77387938,  0.1977879 ], dtype=float32)
+
+    Array values are generated in two parts:
+
+    1. A single value is drawn from the fill strategy and is used to create a
+       filled array.
+    2. Some subset of the coordinates of the array are populated with a value
+       drawn from the elements strategy (or its inferred form).
+
+    You can set ``fill`` to :func:`~hypothesis.strategies.nothing` if you want
+    to disable this behaviour and draw a value for every element.
+
+    By default ``arrays`` will attempt to infer the correct fill behaviour: if
+    ``unique`` is also ``True``, no filling will occur. Otherwise, if it looks
+    safe to reuse the values of elements across multiple coordinates (this will
+    be the case for any inferred strategy, and for most of the builtins, but is
+    not the case for mutable values or strategies built with flatmap, map,
+    composite, etc.) then it will use the elements strategy as the fill, else it
+    will default to having no fill.
+
+    Having a fill helps Hypothesis craft high quality examples, but its
+    main importance is when the array generated is large: Hypothesis is
+    primarily designed around testing small examples. If you have arrays with
+    hundreds or more elements, having a fill value is essential if you want
+    your tests to run in reasonable time.
+    """
 
     infer_xp_is_compliant(xp)
-    check_xp_attributes(xp, ["empty", "full", "all", "isnan", "isfinite", "reshape"])
+    check_xp_attributes(xp, ["zeros", "full", "all", "isnan", "isfinite", "reshape"])
 
     if isinstance(dtype, st.SearchStrategy):
         return dtype.flatmap(
@@ -415,7 +501,15 @@ def array_shapes(
     min_side: int = 1,
     max_side: Optional[int] = None,
 ) -> st.SearchStrategy[Shape]:
-    """Return a strategy for array shapes (tuples of int >= 1)."""
+    """Return a strategy for array shapes (tuples of int >= 1).
+
+    * ``min_dims`` is the smallest length that the generated shape can possess.
+    * ``max_dims`` is the largest length that the generated shape can possess,
+      defaulting to ``min_dims + 2``.
+    * ``min_side`` is the smallest size that a dimension can possess.
+    * ``max_side`` is the largest size that a dimension can possess,
+      defaulting to ``min_side + 5``.
+    """
     check_type(int, min_dims, "min_dims")
     check_type(int, min_side, "min_side")
 
@@ -447,18 +541,15 @@ def check_dtypes(xp, dtypes: List[Type[DataType]], stubs: List[str]):
 
 
 def scalar_dtypes(xp) -> st.SearchStrategy[Type[DataType]]:
-    """Return a strategy for all Array API dtypes."""
+    """Return a strategy for all :array-ref:`valid dtype <data_types.html>` objects."""
     infer_xp_is_compliant(xp)
-
     dtypes, stubs = partition_attributes_and_stubs(xp, DTYPE_NAMES)
     check_dtypes(xp, dtypes, stubs)
-
     return st.sampled_from(dtypes)
 
 
 def boolean_dtypes(xp) -> st.SearchStrategy[Type[Boolean]]:
     infer_xp_is_compliant(xp)
-
     try:
         return st.just(xp.bool)
     except AttributeError:
@@ -468,12 +559,19 @@ def boolean_dtypes(xp) -> st.SearchStrategy[Type[Boolean]]:
         ) from None
 
 
+def numeric_dtypes(xp) -> st.SearchStrategy[Type[Numeric]]:
+    """Return a strategy for all numeric dtype objects."""
+    infer_xp_is_compliant(xp)
+    dtypes, stubs = partition_attributes_and_stubs(xp, NUMERIC_NAMES)
+    check_dtypes(xp, dtypes, stubs)
+    return st.sampled_from(dtypes)
+
+
 def check_valid_sizes(category: str, sizes: Sequence[int], valid_sizes: Sequence[int]):
     invalid_sizes = []
     for size in sizes:
         if size not in valid_sizes:
             invalid_sizes.append(size)
-
     if len(invalid_sizes) > 0:
         f_valid_sizes = ", ".join(str(s) for s in valid_sizes)
         f_invalid_sizes = ", ".join(str(s) for s in invalid_sizes)
@@ -491,25 +589,30 @@ def numeric_dtype_names(base_name: str, sizes: Sequence[int]):
 def integer_dtypes(
     xp, *, sizes: Union[int, Sequence[int]] = (8, 16, 32, 64)
 ) -> st.SearchStrategy[Type[SignedInteger]]:
-    """Return a strategy for signed integer dtypes in the Array API."""
-    infer_xp_is_compliant(xp)
+    """Return a strategy for signed integer dtype objects.
 
+    ``sizes`` contains the signed integer sizes in bits, defaulting to
+    ``(8, 16, 32, 64)`` which covers all valid sizes.
+    """
+    infer_xp_is_compliant(xp)
     if isinstance(sizes, int):
         sizes = (sizes,)
     check_valid_sizes("int", sizes, (8, 16, 32, 64))
-
     dtypes, stubs = partition_attributes_and_stubs(
         xp, numeric_dtype_names("int", sizes)
     )
     check_dtypes(xp, dtypes, stubs)
-
     return st.sampled_from(dtypes)
 
 
 def unsigned_integer_dtypes(
     xp, *, sizes: Union[int, Sequence[int]] = (8, 16, 32, 64)
 ) -> st.SearchStrategy[Type[UnsignedInteger]]:
-    """Return a strategy for unsigned integer dtypes in the Array API."""
+    """Return a strategy for unsigned integer dtype objects.
+
+    ``sizes`` contains the unsigned integer sizes in bits, defaulting to
+    ``(8, 16, 32, 64)`` which covers all valid sizes.
+    """
     infer_xp_is_compliant(xp)
 
     if isinstance(sizes, int):
@@ -527,18 +630,20 @@ def unsigned_integer_dtypes(
 def floating_dtypes(
     xp, *, sizes: Union[int, Sequence[int]] = (32, 64)
 ) -> st.SearchStrategy[Type[Float]]:
-    """Return a strategy for floating dtypes in the Array API."""
-    infer_xp_is_compliant(xp)
+    """Return a strategy for floating-point dtype objects.
 
+    ``sizes`` contains the floating-point sizes in bits, defaulting to
+    ``(32, 64)`` which covers all valid sizes.
+    """
+
+    infer_xp_is_compliant(xp)
     if isinstance(sizes, int):
         sizes = (sizes,)
     check_valid_sizes("int", sizes, (32, 64))
-
     dtypes, stubs = partition_attributes_and_stubs(
         xp, numeric_dtype_names("float", sizes)
     )
     check_dtypes(xp, dtypes, stubs)
-
     return st.sampled_from(dtypes)
 
 
@@ -548,20 +653,41 @@ def valid_tuple_axes(
     min_size: int = 0,
     max_size: Optional[int] = None,
 ) -> st.SearchStrategy[Shape]:
-    """Return a strategy for axes."""
+    """Return a strategy for permissable tuple-values for the ``axis``
+    argument in Array API sequential methods e.g. ``sum``, given the specified
+    dimensionality.
+
+    All tuples will have a length >= ``min_size`` and <= ``max_size``. The default
+    value for ``max_size`` is ``ndim``.
+
+    Examples from this strategy shrink towards an empty tuple, which render most
+    sequential functions as no-ops.
+
+    The following are some examples drawn from this strategy.
+
+    .. code-block:: pycon
+
+      >>> [valid_tuple_axes(3).example() for i in range(4)]
+      [(-3, 1), (0, 1, -1), (0, 2), (0, -2, 2)]
+
+    ``valid_tuple_axes`` can be joined with other strategies to generate
+    any type of valid axis object, i.e. integers, tuples, and ``None``:
+
+    .. code-block:: python
+
+      any_axis_strategy = none() | integers(-ndim, ndim - 1) | valid_tuple_axes(ndim)
+
+    """
     if max_size is None:
         max_size = ndim
-
     check_type(int, ndim, "ndim")
     check_type(int, min_size, "min_size")
     check_type(int, max_size, "max_size")
     order_check("size", 0, min_size, max_size)
     check_valid_interval(max_size, ndim, "max_size", "ndim")
-
     axes = st.integers(0, max(0, 2 * ndim - 1)).map(
         lambda x: x if x < ndim else x - 2 * ndim
     )
-
     return st.lists(
         axes, min_size=min_size, max_size=max_size, unique_by=lambda x: x % ndim
     ).map(tuple)
@@ -660,8 +786,28 @@ def broadcastable_shapes(
     min_side: int = 1,
     max_side: Optional[int] = None,
 ) -> st.SearchStrategy[Shape]:
-    """Return a strategy for generating shapes that are broadcast-compatible
-    with the provided shape."""
+    """Return a strategy for shapes that are broadcast-compatible with the
+    provided shape.
+
+    Examples from this strategy shrink towards a shape with length ``min_dims``.
+    The size of an aligned dimension shrinks towards size ``1``. The size of an
+    unaligned dimension shrink towards ``min_side``.
+
+    * ``shape`` is a tuple of integers.
+    * ``min_dims`` is the smallest length that the generated shape can possess.
+    * ``max_dims`` is the largest length that the generated shape can possess,
+      defaulting to ``min(32, max(len(shape), min_dims) + 2)``.
+    * ``min_side`` is the smallest size that an unaligned dimension can possess.
+    * ``max_side`` is the largest size that an unaligned dimension can possess,
+      defaulting to 2 plus the size of the largest aligned dimension.
+
+    The following are some examples drawn from this strategy.
+
+    .. code-block:: pycon
+
+        >>> [broadcastable_shapes(shape=(2, 3)).example() for i in range(5)]
+        [(1, 3), (), (2, 3), (2, 1), (4, 1, 3), (3, )]
+    """
     check_type(tuple, shape, "shape")
     check_type(int, min_side, "min_side")
     check_type(int, min_dims, "min_dims")
@@ -732,8 +878,42 @@ def mutually_broadcastable_shapes(
     min_side: int = 1,
     max_side: Optional[int] = None,
 ) -> st.SearchStrategy[BroadcastableShapes]:
-    """Return a strategy for generating a specified number of shapes that are
-    mutually-broadcastable with one another and with the provided base shape."""
+    """Return a strategy for a specified number of shapes N that are
+    mutually-broadcastable with one another and with the provided base shape.
+
+    * ``num_shapes`` is the number of mutually broadcast-compatible shapes to generate.
+    * ``base_shape`` is the shape against which all generated shapes can broadcast.
+      The default shape is empty, which corresponds to a scalar and thus does
+      not constrain broadcasting at all.
+    * ``shape`` is a tuple of integers.
+    * ``min_dims`` is the smallest length that the generated shape can possess.
+    * ``max_dims`` is the largest length that the generated shape can possess,
+      defaulting to ``min(32, max(len(shape), min_dims) + 2)``.
+    * ``min_side`` is the smallest size that an unaligned dimension can possess.
+    * ``max_side`` is the largest size that an unaligned dimension can possess,
+      defaulting to 2 plus the size of the largest aligned dimension.
+
+    The strategy will generate a :obj:`python:typing.NamedTuple` containing:
+
+    * ``input_shapes`` as a tuple of the N generated shapes.
+    * ``result_shape`` as the resulting shape produced by broadcasting the N shapes
+      with the base shape.
+
+    The following are some examples drawn from this strategy.
+
+    .. code-block:: pycon
+
+        >>> # Draw three shapes where each shape is broadcast-compatible with (2, 3)
+        ... strat = mutually_broadcastable_shapes(num_shapes=3, base_shape=(2, 3))
+        >>> for _ in range(5):
+        ...     print(strat.example())
+        BroadcastableShapes(input_shapes=((4, 1, 3), (4, 2, 3), ()), result_shape=(4, 2, 3))
+        BroadcastableShapes(input_shapes=((3,), (1,), (2, 1)), result_shape=(2, 3))
+        BroadcastableShapes(input_shapes=((3,), (1, 3), (2, 3)), result_shape=(2, 3))
+        BroadcastableShapes(input_shapes=((), (), ()), result_shape=(2, 3))
+        BroadcastableShapes(input_shapes=((3,), (), (3,)), result_shape=(2, 3))
+
+    """
 
     check_type(int, num_shapes, "num_shapes")
     if num_shapes < 1:
@@ -829,7 +1009,7 @@ class IndexStrategy(st.SearchStrategy):
             result.insert(i, None)
             result_dims += 1
         # Check that we'll have the right number of dimensions; reject if not.
-        # It's easy to do this by construction iff you don't care about shrinking,
+        # It's easy to do this by construction if you don't care about shrinking,
         # which is really important for array shapes.  So we filter instead.
         assume(self.min_dims <= result_dims <= self.max_dims)
         # This is a quick-and-dirty way to insert ..., xor shorten the indexer,
@@ -854,20 +1034,40 @@ class IndexStrategy(st.SearchStrategy):
 def indices(
     shape: Shape,
     *,
-    min_dims: int = 0,
+    min_dims: int = 1,
     max_dims: Optional[int] = None,
     allow_ellipsis: bool = True,
     allow_none: bool = False,
 ) -> st.SearchStrategy[BasicIndex]:
-    """Return a strategy for indices."""
+    """Return a strategy for :array-ref:`valid indices <indexing.html>` of
+    arrays with the specified shape.
+
+    It generates tuples containing some mix of integers, :obj:`python:slice`
+    objects, ``...`` (an ``Ellipsis``), and ``None``. When a length-one tuple
+    would be generated, this strategy may instead return the element which will
+    index the first axis, e.g. ``5`` instead of ``(5,)``.
+
+    * ``shape`` is the shape of the array that will be indexed, as a tuple of
+      integers >= 0. This must be at least two-dimensional for a tuple to be a
+      valid index; for one-dimensional arrays use
+      :func:`~hypothesis.strategies.slices` instead.
+    * ``min_dims`` is the minimum dimensionality of the resulting array from use of
+      the generated index.  If ``min_dims == 0``, zero-dimensional arrays are allowed.
+    * ``max_dims`` is the the maximum dimensionality of the resulting array,
+      defaulting to ``max(len(shape), min_dims) + 2``.
+    * ``allow_ellipsis`` specifies whether ``...`` is allowed in the index.
+    * ``allow_none`` specifies whether ``None`` is allowed in the index.
+    """
     check_type(tuple, shape, "shape")
+    if len(shape) == 0:
+        raise InvalidArgument("No valid indices for zero-dimensional arrays")
     check_type(bool, allow_ellipsis, "allow_ellipsis")
     check_type(bool, allow_none, "allow_none")
     check_type(int, min_dims, "min_dims")
     if max_dims is None:
         max_dims = min(max(len(shape), min_dims) + 2, 32)
     check_type(int, max_dims, "max_dims")
-    order_check("dims", 0, min_dims, max_dims)
+    order_check("dims", 1, min_dims, max_dims)
     if not all(isinstance(x, int) and x >= 0 for x in shape):
         raise InvalidArgument(
             f"shape={shape!r}, but all dimensions must be of integer size >= 0"
@@ -882,7 +1082,29 @@ def indices(
 
 
 def get_strategies_namespace(xp) -> SimpleNamespace:
-    """Creates a strategies namespace."""
+    """Creates a strategies namespace for the passed array module.
+
+    * ``xp`` is the Array API library to automatically pass to the namespaced methods.
+
+    A :obj:`python:types.SimpleNamespace` is returned which contains all the
+    strategy methods in this module but without requiring the ``xp`` argument.
+
+    Creating and using a strategies namespace for NumPy's Array API implemention
+    would go like this:
+
+    .. code-block:: pycon
+
+      >>> from numpy import array_api as xp
+      >>> xps = get_strategies_namespace(xp)
+      >>> x = xps.arrays(xp.int8, (2, 3)).example()
+      >>> x
+      Array([[-8,  6,  3],
+             [-6,  4,  6]], dtype=int8)
+      Array([-8, 6, 3], dtype=int8)
+      >>> x.__array_namespace__() is xp
+      True
+
+    """
     infer_xp_is_compliant(xp)
 
     return SimpleNamespace(
@@ -899,6 +1121,9 @@ def get_strategies_namespace(xp) -> SimpleNamespace:
         boolean_dtypes=update_wrapper(
             lambda *a, **kw: boolean_dtypes(xp, *a, **kw), boolean_dtypes
         ),
+        numeric_dtypes=update_wrapper(
+            lambda *a, **kw: numeric_dtypes(xp, *a, **kw), numeric_dtypes
+        ),
         integer_dtypes=update_wrapper(
             lambda *a, **kw: integer_dtypes(xp, *a, **kw), integer_dtypes
         ),
